@@ -20,6 +20,9 @@ limitations under the License.
 """
 from __future__ import annotations
 
+import asyncio
+import dataclasses
+import json
 from typing import (
     TYPE_CHECKING,
     Dict,
@@ -29,22 +32,19 @@ from typing import (
     Union,
 )
 
-from collections import defaultdict
 from pyee import AsyncIOEventEmitter    # type: ignore
 from wechaty_puppet import (    # type: ignore
     ContactGender,
     ContactPayload,
     ContactQueryFilter,
-    ContactType
+    ContactType,
+
+    get_logger,
+    FileBox
 )
 # from wechaty.utils import type_check
 
 from ..accessory import Accessory
-# from wechaty import Accessory
-from ..config import (
-    FileBox,
-    get_logger,
-)
 
 
 if TYPE_CHECKING:
@@ -114,6 +114,7 @@ class Contact(Accessory, AsyncIOEventEmitter):
         :return:
         """
         log.info('find() <%s, %s>', cls, query)
+
         contact_list = await cls.find_all(query)
         if len(contact_list) == 0:
             return None
@@ -133,6 +134,9 @@ class Contact(Accessory, AsyncIOEventEmitter):
         contact_ids = await cls.get_puppet().contact_list()
         contacts = [Contact.load(contact_id) for contact_id in contact_ids]
 
+        # load contact parallel using asyncio.gather method
+        await asyncio.gather(*[contact.ready() for contact in contacts])
+
         # async load
         batch_size = 16
         # slice contacts by batch_size
@@ -191,7 +195,7 @@ class Contact(Accessory, AsyncIOEventEmitter):
             identity = self.contact_id
         else:
             identity = 'loading ...'
-        return 'Contact <%s>' % identity
+        return 'Contact <%s> <%s>' % (self.contact_id, identity)
 
     async def say(self, message: Union[str, Message, FileBox, Contact, UrlLink]
                   ) -> Optional[Message]:
@@ -202,6 +206,9 @@ class Contact(Accessory, AsyncIOEventEmitter):
         if not self.is_ready():
             await self.ready()
 
+        # import some class because circular dependency
+        from wechaty.user.url_link import UrlLink
+
         if isinstance(message, str):
             # say text
             msg_id = await self.puppet.message_send_text(
@@ -224,7 +231,7 @@ class Contact(Accessory, AsyncIOEventEmitter):
             # use this way to resolve circulation dependency import
             msg_id = await self.puppet.message_send_url(
                 conversation_id=self.contact_id,
-                url=message.url
+                url=json.dumps(dataclasses.asdict(message.payload))
             )
         # elif isinstance(message, MiniProgram):
         #     msg_id = await self.puppet.message_send_mini_program(
@@ -263,20 +270,20 @@ class Contact(Accessory, AsyncIOEventEmitter):
         if self.payload is None:
             raise Exception('can"t load contact payload <%s>' % self)
 
-        if new_alias is None:
-            await self.ready()
-            if self.payload.alias is None:
-                return ''
-            return self.payload.alias
+        # if new_alias is None:
+        #     if self.payload.alias is None:
+        #         return ''
+        #     return self.payload.alias
 
         try:
-            await self.puppet.contact_alias(self.contact_id, new_alias)
+            alias = await self.puppet.contact_alias(self.contact_id, new_alias)
             await self.ready(force_sync=True)
             if new_alias != self.payload.alias:
                 log.info(
                     'Contact alias(%s) sync with server fail: \
                     set(%s) is not equal to get(%s)',
                     new_alias, new_alias, self.payload.alias)
+            return alias
         # pylint:disable=W0703
         except Exception as exception:
             log.info(
@@ -352,7 +359,7 @@ class Contact(Accessory, AsyncIOEventEmitter):
             return None
         return self.payload.city
 
-    async def avatar(self, file_box: Optional[FileBox]) -> FileBox:
+    async def avatar(self, file_box: Optional[FileBox] = None) -> FileBox:
         """
         get the avatar of the account
         """
@@ -374,7 +381,7 @@ class Contact(Accessory, AsyncIOEventEmitter):
         """
         sync the contact data
         """
-        return self.ready()
+        await self.ready()
 
     def is_self(self) -> bool:
         """
